@@ -14,6 +14,7 @@ CURRENT_LINK = 'current'
 FAILSAFE_NAME = 'failsafe'
 PROTECTED = [CURRENT_LINK,FAILSAFE_NAME]
 DEFAULT_FIRMWARE_DIRECTORY = '/opt/firmware'
+LOG_FORMAT = '[%(levelname)s] %(asctime)s -- %(message)s'
 
 def clean( target=DEFAULT_FIRMWARE_DIRECTORY, protected = None ):
     """Naive cleaning implementation
@@ -40,8 +41,18 @@ def clean( target=DEFAULT_FIRMWARE_DIRECTORY, protected = None ):
     return current_target
 
 def final_path( link ):
-    """Get the final path of the given link"""
-    return os.path.normpath( os.path.realpath( link ) )
+    """Get the final path of the given link
+    
+    raises IOError if link target does not exist, or the target is not a directory 
+    
+    returns normalized real target path of the link
+    """
+    real = os.path.normpath( os.path.realpath( link ) )
+    if not os.path.exists( real ):
+        raise IOError( "Target of link %(link)r (%(real)s) does not exist"%locals())
+    if not os.path.isdir( real ):
+        raise IOError( "Target of link %(link)r is not a directory"%locals())
+    return real 
 
 def swap_link( final_target, current ):
     """Swap current link to point to final_target
@@ -140,6 +151,7 @@ def enable( final_target, current ):
             pipe = nbio.Process( [
                 pre_install,
                 final_target,
+                'False'
             ], by_line=True, stderr=-1) | report_progress
             pipe()
         log.info( 'Setting firmware current' )
@@ -157,9 +169,25 @@ def enable( final_target, current ):
         pipe = nbio.Process([
             post_install,
             final_target,
+            'True',
         ], by_line=True, stderr=-1) | report_progress
         pipe()
-    
+
+def ensure_current_link( current, failsafe ):
+    """Ensure that current is a link (not a directory)"""
+    if (
+        os.path.isdir( current ) and 
+        not os.path.islink( current)
+    ):
+        if not os.path.exists(failsafe):
+            # this is the initial install case, where we want failsafe to be 
+            # created from the initial installation image...
+            os.rename( current, failsafe )
+            os.symlink( failsafe, current )
+        else:
+            raise RuntimeError(
+                """%(current)r is a directory, it should be a link.  Failsafe already exists."""%locals()
+            )
 
 def install( filename, keyring='/etc/fussy/keys', target='/opt/firmware' ):
     """Install given firmware <filename> into given target directory
@@ -176,11 +204,13 @@ def install( filename, keyring='/etc/fussy/keys', target='/opt/firmware' ):
     current = os.path.join( os.path.normpath( target ), CURRENT_LINK )
     failsafe = os.path.join( os.path.normpath( target ), FAILSAFE_NAME )
     
+    ensure_current_link( current, failsafe )
+    
     previous = None
-    if os.path.exists( current ):
+    try:
         previous = final_path( current )
-        if not os.path.exists( previous ):
-            previous = None
+    except IOError, err:
+        log.warn( "Target of current does not appear to exist: %s", err )
     if not previous and os.path.exists( failsafe ) :
         previous = failsafe
     log.info( 'Previous installation: %s', previous )
@@ -191,7 +221,7 @@ def install( filename, keyring='/etc/fussy/keys', target='/opt/firmware' ):
     try:
         enable( final_target, current )
     except Exception, err:
-        log.error( "Failure installing %s: %s", err )
+        log.error( "Failure installing %s: %s", final_target, err )
         log.error( "Traceback: %s", traceback.format_exc())
         if previous:
             log.warn( "Attempting to restore previous: %s", previous )
@@ -239,6 +269,13 @@ def get_options():
     )
     return parser
 
+def configure_log( logfile ):
+    if logfile:
+        logging.basicConfig(filename=logfile,level=logging.INFO, format=LOG_FORMAT)
+    else:
+        logging.basicConfig(level=logging.INFO, format=LOG_FORMAT)
+    return logfile
+
 def main():
     """Main entry-point for the fussy-install script
     
@@ -254,12 +291,10 @@ def main():
             options.file = args[0]
         else:
             parser.error( "Need a file to install" )
-    if options.logfile:
-        logging.basicConfig(filename=options.logfile,level=logging.INFO)
-    else:
-        logging.basicConfig(level=logging.INFO)
+    configure_log( options.logfile )
     try:
-        return install( options.file, options.keyring, options.target )
+        installed = install( options.file, options.keyring, options.target )
+        return 0
     except Exception, err:
         log.error(
             "Failure during installation: %s", err,
@@ -286,6 +321,13 @@ def clean_main():
         type="string",
         help="Directory into which to rsync the firmware (default /opt/firmware)",
     )
+    parser.add_option(
+        '-l', '--logfile',
+        dest='logfile',
+        default='/tmp/fussy-install.log',
+        help = "File into which to write the fussy installation log (default /tmp/fussy-install.log)",
+    )
     options,args = parser.parse_args()
+    configure_log( options.logfile )
     clean( options.target )
     return 0
